//
//  NetThumbButton.m
//  CAEmitterLayerDemo
//
//  Created by 张锋 on 16/6/28.
//  Copyright © 2016年 张锋. All rights reserved.
//

#import "NetThumbButton.h"
#import "EmitterLayer.h"

#define dispatch_main_sync_safe(block)\
    if ([NSThread isMainThread]) {\
        block();\
    } else {\
        dispatch_sync(dispatch_get_main_queue(), block);\
    }

#define dispatch_main_async_safe(block)\
    if ([NSThread isMainThread]) {\
        block();\
    } else {\
        dispatch_async(dispatch_get_main_queue(), block);\
    }

@interface NetThumbButton ()

{
    BOOL _selected;
}
@property (nonatomic, strong) EmitterLayer *emiLayer;
@end

@implementation NetThumbButton

- (instancetype)init
{
    CGRect frame = CGRectMake(0, 0, 30, 30);
    self = [self initWithFrame:frame];
    if (self) {
    }
    return self;
}

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self setupSubViews];
    }
    return self;
}

- (instancetype)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self setupSubViews];
    }
    return self;
}

- (void)setupSubViews
{
    _selected = NO;
    [self insertSubview:self.emiLayer atIndex:0];
    [self setImage:[UIImage imageNamed:@"Like-PlaceHold"] forState:UIControlStateNormal];
    [self addTarget:self action:@selector(thumbButtonClick) forControlEvents:UIControlEventTouchUpInside];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(thumbButtonFailed) name:@"kThumbButtonHasFailWithThumb" object:nil];
}

- (EmitterLayer *)emiLayer
{
    if (!_emiLayer) {
        _emiLayer = [[EmitterLayer alloc] initWithFrame:self.bounds];
    }
    return _emiLayer;
}

- (void)setHighlighted:(BOOL)highlighted
{
    //重写点击高亮效果，这样动画看起来更流畅
}

#pragma mark - 点赞
- (void)thumbButtonClick
{
    if (self.buttonClick) {
        self.buttonClick();
    }
    
    if (self.delegate && [self.delegate respondsToSelector:@selector(thumbButtonClick)]) {
        [self.delegate thumbButtonClick];
    }
    
    __weak typeof(self)weakSelf = self;
    self.transform = CGAffineTransformIdentity;
    
    if (_selected == NO) {
        [self.emiLayer animation];
        
        [self setImage:[UIImage imageNamed:@"Like-Blue"] forState:UIControlStateNormal];
        
        [UIView animateWithDuration:0.8f animations:^{
            __strong typeof(weakSelf)strongSelf = weakSelf;
            dispatch_main_sync_safe(^{
                strongSelf.transform = CGAffineTransformMakeScale(2.0f, 2.0f);
                strongSelf.alpha = 0;
            });
        } completion:^(BOOL finished) {
            _selected = YES;
            dispatch_time_t delay = dispatch_time(DISPATCH_TIME_NOW, 0.2 * NSEC_PER_SEC);
            dispatch_after(delay, dispatch_get_main_queue(), ^{
                __strong typeof(weakSelf)strongSelf = weakSelf;
                dispatch_main_sync_safe(^{
                    strongSelf.transform = CGAffineTransformIdentity;
                    strongSelf.alpha = 1;
                });
            });
        }];
    }else {
        [self setImage:[UIImage imageNamed:@"Like-PlaceHold"] forState:UIControlStateNormal];
         _selected = NO;
    }
}

#pragma mark - 点赞失败
- (void)thumbButtonFailed
{
    if (_selected == NO) return;
    [self setImage:[UIImage imageNamed:@"Like-PlaceHold"] forState:UIControlStateNormal];
    _selected = NO;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"kThumbButtonHasFailWithThumb" object:nil];
}

@end
